@@ -13,6 +13,82 @@
 //! go. Treating enums as a transparent layer can make our furture implementation compatiable 
 //! with ETH.
 //! 
+//! ### Design principle
+//! 
+//! Accroding to the ETH Yellow Paper, all supported data structure can be represented with 
+//! either recursive list of byte arrays ![](https://latex.codecogs.com/svg.latex?\mathbb{L}) or 
+//! byte arrays ![](https://latex.codecogs.com/svg.latex?\mathbb{B}). So we can transform all 
+//! Rust's compound types, for example, tuple, struct and list, into lists. And then encode them 
+//! as exactly described in the paper
+//! 
+//! For example, the structure in example code, can be internally treated as the following form:
+//! 
+//! ```
+//! [
+//!     "This is a tooooooooooooo loooooooooooooooooooong tag", 
+//!     [
+//!         114514, 
+//!         [191, -9810], 
+//!         [
+//!             [[], [[]], [[], [[]]]]
+//!         ]
+//!     ], 
+//!     "哼.啊啊啊啊啊啊啊啊啊啊啊啊啊啊啊啊啊啊"
+//! ]
+//! ```
+//! 
+//! ### ZST serialization
+//! 
+//! In Rust, we can represent 'empty' in many ways, for example:
+//! 
+//! ```
+//! [], (), "", b"", struct Empty, Variant::Empty, None, PhantomData<T>
+//! ```
+//! 
+//! In out implementation:
+//! 
+//! 1. `[]` and `()` are considered empty list, thus should be serialized into 0xc0
+//! 2. All other ZSTs are considered empty, thus should be serialized into 0x80
+//! 
+//! To better understand ZSTs' behavior when serializing, try this code:
+//! 
+//! ```rust
+//! #[test]
+//! fn test_compound_zst() {
+//!     #[derive(Serialize, Debug, PartialEq, Eq)]
+//!     struct ZST;
+//! 
+//!     #[derive(Serialize, Debug, PartialEq, Eq)]
+//!     enum Simple {
+//!         Empty(ZST),
+//!         #[allow(dead_code)]
+//!         Int((u32, u64))
+//!     }
+//! 
+//!     #[derive(Serialize, Debug, PartialEq, Eq)]
+//!     struct ContainZST(Simple);
+//! 
+//!     #[derive(Serialize, Debug, PartialEq, Eq)]
+//!     struct StructZST {
+//!         zst: Simple
+//!     }
+//! 
+//!     let zst = Simple::Empty(ZST);
+//!     let zst_res = to_bytes(&zst).unwrap();
+//!     assert_eq!(zst_res, [0x80]);
+//! 
+//!     let with_zst = ContainZST(Simple::Empty(ZST));
+//!     let with_zst_res = to_bytes(&with_zst).unwrap();
+//!     // the container is transparent because its a newtype
+//!     assert_eq!(with_zst_res, [0x80]);
+//!     
+//!     let with_zst = StructZST { zst: Simple::Empty(ZST) };
+//!     let with_zst_res = to_bytes(&with_zst).unwrap();
+//!     // the container is a list, to this is equivlent to [""]
+//!     assert_eq!(with_zst_res, [0xc1, 0x80]);
+//!     }
+//! ```
+//! 
 //! ### Example code
 //! 
 //! You can find more examples [here](https://github.com/M4tsuri/serlp/tree/main/example)
@@ -64,42 +140,57 @@
 //! }
 //! ```
 //! 
-//! ### Design principle
-//! 
-//! Accroding to the ETH Yellow Paper, all supported data structure can be represented with 
-//! either recursive list of byte arrays ![](https://latex.codecogs.com/svg.latex?\mathbb{L}) or 
-//! byte arrays ![](https://latex.codecogs.com/svg.latex?\mathbb{B}). So we can transform all 
-//! Rust's compound types, for example, tuple, struct and list, into lists. And then encode them 
-//! as exactly described in the paper
-//! 
-//! For example, the structure in example code, can be internally treated as the following form:
-//! 
-//! ```
-//! [
-//!     "This is a tooooooooooooo loooooooooooooooooooong tag", 
-//!     [
-//!         114514, 
-//!         [191, -9810], 
-//!         [
-//!             [[], [[]], [[], [[]]]]
-//!         ]
-//!     ], 
-//!     "哼.啊啊啊啊啊啊啊啊啊啊啊啊啊啊啊啊啊啊"
-//! ]
-//! ```
 
-mod ser;
+pub mod ser;
 pub mod error;
-mod de;
+pub mod tree_de;
 pub mod rlp;
+pub mod de;
 
 #[cfg(test)]
 mod test {
     use serde::{Serialize, Deserialize};
     use serde_bytes::Bytes;
 
+    use crate::de::RlpProxy;
+    use crate::rlp::RlpTree;
     use crate::rlp::to_bytes;
     use crate::rlp::from_bytes;
+    use crate::tree_de::from_rlp_tree;
+
+    #[test]
+    fn test_proxy() {
+        #[derive(Serialize, Deserialize, PartialEq, Eq, Debug)]
+        #[serde(from = "RlpProxy")]
+        enum Classify {
+            Zero(u8),
+            One(u8),
+            Ten((u8, u8))
+        }
+        
+        impl From<RlpProxy> for Classify {
+            fn from(proxy: RlpProxy) -> Self {
+                let raw = proxy.raw();
+                let mut tree = proxy.rlp_tree();
+                if tree.value_count() == 2 {
+                    return Classify::Ten(from_bytes(raw).unwrap())
+                }
+
+                let val = tree.next().unwrap()[0];
+                match val {
+                    0 => Classify::Zero(0),
+                    1 => Classify::One(1),
+                    _ => panic!("Value Error.")
+                }
+            }
+        }
+
+        let value = Classify::Ten((12, 34));
+        let encoded = to_bytes(&value).unwrap();
+        let origin = from_bytes(&encoded).unwrap();
+
+        assert_eq!(value, origin);
+    }
 
     #[test]
     fn test_long_string() {
@@ -246,6 +337,7 @@ mod test {
     fn test_simple_enum() {
         #[derive(Serialize, Debug, PartialEq, Eq)]
         enum Simple {
+            #[allow(dead_code)]
             Empty,
             Int(u32)
         }
@@ -254,21 +346,34 @@ mod test {
         struct Equiv(u32);
 
         let simple_enum = Simple::Int(114514);
-        let unit_variant = Simple::Empty;
         let equiv = Equiv(114514);
 
         let enum_res = to_bytes(&simple_enum).unwrap();
-        let empty_res = to_bytes(&unit_variant).unwrap();
         let struct_res = to_bytes(&equiv).unwrap();
 
         assert_eq!(enum_res, struct_res);
-        assert_eq!(empty_res, []);
+    }
+
+    #[test]
+    fn test_unit_variant() {
+        #[derive(Serialize, Debug, PartialEq, Eq)]
+        enum Simple {
+            Empty,
+            #[allow(dead_code)]
+            Int(u32)
+        }
+
+        let unit_variant = Simple::Empty;
+        let empty_res = to_bytes(&unit_variant).unwrap();
+
+        assert_eq!(empty_res, [0x80]);
     }
 
     #[test]
     fn test_variant_tuple() {
         #[derive(Serialize, Debug, PartialEq, Eq)]
         enum Simple {
+            #[allow(dead_code)]
             Empty,
             Int((u32, u64))
         }
@@ -277,15 +382,47 @@ mod test {
         struct Equiv((u32, u64));
 
         let simple_enum = Simple::Int((114514, 1919810));
-        let unit_variant = Simple::Empty;
         let equiv = Equiv((114514, 1919810));
 
         let enum_res = to_bytes(&simple_enum).unwrap();
-        let empty_res = to_bytes(&unit_variant).unwrap();
         let struct_res = to_bytes(&equiv).unwrap();
 
         assert_eq!(enum_res, struct_res);
-        assert_eq!(empty_res, []);
+    }
+
+    #[test]
+    fn test_compound_zst() {
+        #[derive(Serialize, Debug, PartialEq, Eq)]
+        struct ZST;
+
+        #[derive(Serialize, Debug, PartialEq, Eq)]
+        enum Simple {
+            Empty(ZST),
+            #[allow(dead_code)]
+            Int((u32, u64))
+        }
+
+        #[derive(Serialize, Debug, PartialEq, Eq)]
+        struct ContainZST(Simple);
+
+        #[derive(Serialize, Debug, PartialEq, Eq)]
+        struct StructZST {
+            zst: Simple
+        }
+
+        let zst = Simple::Empty(ZST);
+        let zst_res = to_bytes(&zst).unwrap();
+        assert_eq!(zst_res, [0x80]);
+
+        let with_zst = ContainZST(Simple::Empty(ZST));
+        let with_zst_res = to_bytes(&with_zst).unwrap();
+        // the container is transparent because its a newtype
+        assert_eq!(with_zst_res, [0x80]);
+        
+        let with_zst = StructZST { zst: Simple::Empty(ZST) };
+        let with_zst_res = to_bytes(&with_zst).unwrap();
+        // the container is a list, to this is equivlent to [""]
+        assert_eq!(with_zst_res, [0xc1, 0x80]);
     }
 
     #[test]
@@ -340,6 +477,34 @@ mod test {
         assert_eq!(simple_res, struct_res);
     }
 
+    #[test]
+    fn test_zst_struct() {
+        #[derive(Serialize, Debug, PartialEq, Eq, Deserialize)]
+        struct ZST;
+
+        #[derive(Serialize, Debug, PartialEq, Eq, Deserialize)]
+        struct WithZST {
+            f1: u8,
+            f2: ZST,
+            f3: (),
+            f4: u8,
+        }
+
+        let zst = WithZST {
+            f1: 1,
+            f2: ZST,
+            f3: (),
+            f4: 4
+        };
+
+        let encoded = to_bytes(&zst).unwrap();
+        let origin: WithZST = from_bytes(&encoded).unwrap();
+        let expected = [0xc4, 0x1, 0x80, 0xc0, 0x4];
+        
+        assert_eq!(origin, zst);
+        assert_eq!(encoded, expected);
+    }
+
     #[test]
     fn test_embedded_struct() {
         #[derive(Serialize, Debug, PartialEq, Eq, Deserialize)]
@@ -373,8 +538,11 @@ mod test {
         };
 
         let encode = to_bytes(&embed).unwrap();
+        let tree = RlpTree::new(&encode).unwrap();
         let origin: Embeding = from_bytes(&encode).unwrap();
-        assert_eq!(embed, origin)
+        let origin_with_tree = from_rlp_tree(tree).unwrap();
+        assert_eq!(embed, origin);
+        assert_eq!(embed, origin_with_tree);
     }
 
 }
