@@ -1,7 +1,11 @@
-use serde::{ser::{self, SerializeTuple}, Serialize};
-use crate::error::{Error, Result};
+use serde::{
+    ser::{self, SerializeTuple}, 
+    Serialize
+};
 use paste::paste;
 
+use crate::error::{Error, Result};
+
 pub struct Serializer {
     /// the parser stack, we simulate recursion with this structure
     stack: Vec<Vec<u8>>
@@ -119,6 +123,8 @@ impl<'a> ser::Serializer for &'a mut Serializer {
     /// So what is the difference between (), (()), None, "" and []
     /// none just means nothing, it not even an empty list
     fn serialize_none(self) -> Result<()> {
+        let last = self.stack.last_mut().unwrap();
+        last.push(0x80);
         Ok(())
     }
 
